#ifndef LIBKYPROCESS_H
#define LIBKYPROCESS_H

#endif

/**
 * @file libkyprocess.h
 * @author shaozhimin (shaozhimin@kylinos.cn)
 * @brief 某一进程的瞬时详细信息
 * @version 0.1
 * @date 2022-09-13
 * 
 * @copyright Copyright (c) 2022
 * @defgroup 某一进程的瞬时详细信息
 * @{
 * 
 */

#ifdef __cplusplus
extern "C" {
#endif

#define PRG_SOCKET_PFX    "socket:["
#define PRG_SOCKET_PFXl (strlen(PRG_SOCKET_PFX))
#define PRG_SOCKET_PFX2   "[0000]:"
#define PRG_SOCKET_PFX2l  (strlen(PRG_SOCKET_PFX2))

#define LINE_MAX 4096

#define COLWID 240 
#define UPTIME_FILE  "/proc/uptime"
#define MEMINFO					"/proc/meminfo"
#define MAX_PF_NAME				1024
#define PROC_DIR                "/proc"
#define MAX_PROC_NUM            1024
#define PROC_NUM                500
#define BUF_SIZE 1024
static int uptime_fd = -1;
#define LC_NUMERIC        __LC_NUMERIC
#define __LC_NUMERIC		 1
static char buf[8192];
#define O_RDONLY	     00
#define BAD_OPEN_MESSAGE					\
"Error: /proc must be mounted\n"				\
"  To mount /proc at boot you need an /etc/fstab line like:\n"	\
"      proc   /proc   proc    defaults\n"			\
"  In the meantime, run \"mount proc /proc -t proc\"\n"

#define FILE_TO_BUF(filename, fd) do{				\
    static int local_n;						\
    if (fd == -1 && (fd = open(filename, O_RDONLY)) == -1) {	\
	fputs(BAD_OPEN_MESSAGE, stderr);			\
	fflush(NULL);						\
	_exit(102);						\
    }								\
    lseek(fd, 0L, SEEK_SET);					\
    if ((local_n = read(fd, buf, sizeof buf - 1)) < 0) {	\
	perror(filename);					\
	fflush(NULL);						\
	_exit(103);						\
    }								\
    buf[local_n] = '\0';					\
}while(0)

#define SET_IF_DESIRED(x,y) do{  if(x) *(x) = (y); }while(0)

struct pstat {
	long unsigned int utime_ticks;
	long unsigned int stime_ticks;
	long unsigned int cpu_total_time;
};

/* 进程信息 */
typedef struct process_info
{
    long proc_pid; //进程id
    char proc_name[128]; //进程名
    char proc_command[4096]; //进程启动的命令行,包含启动参数
    char proc_uid[128]; //进程所属用户　用户uid
    char start_time[128]; //进程启动的时间 (char * ctime(time_t time) 函数返回值Tue Jul 28 08:50:26 2020)
    unsigned long utime; //进程cpu使用时间,单位s
    double proc_cpu; //进程cpu使用率 %
    double proc_mem; //进程mem使用量 %
    char proc_state[20]; //进程状态
    char proc_time[20]; //进程运行时间
    char proc_port[20];  //进程端口号
       
} process_info;

#define PROCESS_ITEM 14
 
typedef struct {
	unsigned long user;
	unsigned long nice;
	unsigned long system;
	unsigned long idle;
}Total_Cpu_Occupy_t;
 
 
typedef struct {
	unsigned int pid;
	unsigned long utime;  //user time
	unsigned long stime;  //kernel time
	unsigned long cutime; //all user time
        unsigned long cstime; //all dead time
}Proc_Cpu_Occupy_t;

/**
 * @brief 获取某一进程的CPU使用率
 * 
 * @param proc_num 进程号
 * @return float CPU使用率
 */
extern float kdk_get_process_cpu_usage_percent(int proc_num);

/**
 * @brief 获取某一进程的内存占用率
 * 
 * @param proc_num 进程号
 * @return float 内存占用率
 */
extern float kdk_get_process_mem_usage_percent(int proc_num);

/**
 * @brief 获取某一进程的进程状态
 * 
 * @param proc_num 进程号
 * @return char* 进程状态
 */
extern char* kdk_get_process_status(int proc_num);

/**
 * @brief 获取某一进程的端口号占用
 * 
 * @param proc_num 进程号
 * @return char* 使用的端口号
 */
extern int kdk_get_process_port(int proc_num);

/**
 * @brief 获取某一进程的启动时间
 * 
 * @param proc_num 进程号
 * @return char* 启动时间
 */
extern char* kdk_get_process_start_time(int proc_num);

/**
 * @brief 获取某一进程的运行时间
 * 
 * @param proc_num 进程号
 * @return char* 运行时间
 */
extern char* kdk_get_process_running_time(int proc_num);

/**
 * @brief 获取某一进程的cpu时间
 * 
 * @param proc_num 进程号
 * @return char* cpu时间
 */
extern char* kdk_get_process_cpu_time(int proc_num);

/**
 * @brief 获取某一进程的Command
 * 
 * @param proc_num 进程号
 * @return char* Command
 */
extern char* kdk_get_process_command(int proc_num);

/**
 * @brief 获取某一进程的属主
 * 
 * @param proc_num 进程号
 * @return char* 属主
 */
extern char* kdk_get_process_user(int proc_num);


/**
 * @brief 获取某一进程的CPU使用率
 * 
 * @param proc_name 进程名
 * @return float CPU使用率
 */
extern float kdk_procname_get_process_cpu_usage_percent(char *proc_name);

/**
 * @brief 获取某一进程的IO使用率
 * 
 * @param proc_name 进程名
 * @return float IO使用率
 */
extern float kdk_procname_get_process_io_usage_percent(char *proc_name);

/**
 * @brief 获取某一进程的信息
 * 
 * @param proc_name 进程名
 * @return float 内存占用率
 */
extern char** kdk_procname_get_process_infomation(char *proc_name);

/**
 * @brief 获取进程所有信息
 *
 * @return char** 进程所有信息
 */
extern char** kdk_get_process_all_information();

/**
 * @brief 用于回收字符串列表
 * 
 * @param ptr 字符串列表
 */
extern inline void kdk_proc_freeall(char **ptr);


#ifdef __cplusplus
}

#endif
/**
  * @}
  */