#include "libkyrealtimeinfo.h"
#include "fcntl.h"
#include "stdio.h"
#include "net/if.h"
#include "unistd.h"
#include "stdlib.h"
#include "string.h"
#include "ifaddrs.h"
#include "sys/types.h"
#include "smartctl.h"
using namespace KDK_SYSTEM;

RealTimeInfo::RealTimeInfo()
{

}

float RealTimeInfo::kdk_real_get_io_speed()
{
    return 0;
}

float RealTimeInfo::kdk_real_get_net_speed()
{
    struct if_speed ndev;

    int ret = 0;
    memset(&ndev,0,sizeof(ndev));
    sprintf(ndev.ifs_name,"enp3s0");
    ndev.ifs_us = 100000;
    ret = get_if_speed(&ndev);
    if(ret < 0)
        return -1;
    return ndev.ifs_ospeed * 1.0/(ndev.ifs_us/1000 * 0.001) / 1024.0f;
}

float RealTimeInfo::kdk_real_get_cpu_temperature()
{
    char tmp[32] = "\0";
    char path[64] = "\0";
    for(int i = 1; i; i++)
    {
        memset(path,0,64);
        snprintf(path,64,"/sys/class/hwmon/hwmon%d",i);
        if (0 != access(path, F_OK))
        {
            snprintf(path,64,"/sys/class/hwmon/hwmon1/temp1_input");
            break;
        }
        snprintf(path,64,"/sys/class/hwmon/hwmon%d/name",i);
        if (0 != access(path, F_OK))
            continue;
        else
        snprintf(path,64,"/sys/class/hwmon/hwmon%d/temp1_input",i);
    }
    
    FILE *fp = fopen(path,"r");
    if(fp)
    {
        fgets(tmp,sizeof(tmp),fp);
        fclose(fp);
        return atof(tmp) / 1000;
    }
    return -1;
}

float RealTimeInfo::kdk_real_get_disk_temperature(const char *name)
{
    nvmeTemp = "";
    sataTemp = "";
    if(0 == main_worker(name))
    {
        printf("Temp %s\n",nvmeTemp.empty() ? sataTemp.c_str() : nvmeTemp.c_str());
        return atof(nvmeTemp.empty() ? sataTemp.c_str() : nvmeTemp.c_str());
    }
    return -1;
}

int RealTimeInfo::get_if_dbytes(struct if_info *ndev)
{
    if(!ndev)
        return -1;

    struct ifaddrs *ifa_list = NULL;

    struct ifaddrs *ifa = NULL;

    struct if_data *ifd = NULL;

    int ret = 0;

    ret = getifaddrs(&ifa_list);

    if (ret < 0)
    {

        perror("Get Interface Address Fail:");

        goto end;
    }

    for (ifa = ifa_list; ifa; ifa = ifa->ifa_next)
    {

        if (!(ifa->ifa_flags & IFF_UP) && !(ifa->ifa_flags & IFF_RUNNING))

            continue;

        if (ifa->ifa_data == 0)

            continue;

        ret = strcmp(ifa->ifa_name, ndev->ifi_name);

        if (ret == 0)
        {

            ifd = (struct if_data *)ifa->ifa_data;

            ndev->ifi_ibytes = ifd->ifi_ibytes;

            ndev->ifi_obytes = ifd->ifi_obytes;

            break;
        }
    }

    freeifaddrs(ifa_list);

end:
    return (ret ? -1 : 0);
}
int RealTimeInfo::get_if_speed(struct if_speed *ndev)
{
    if(!ndev)
        return -1;

    struct if_info *p1 = NULL, *p2 = NULL;

    p1 = (struct if_info *)malloc(sizeof(struct if_info));

    p2 = (struct if_info *)malloc(sizeof(struct if_info));

    bzero(p1, sizeof(struct if_info));

    bzero(p2, sizeof(struct if_info));

    strncpy(p1->ifi_name, ndev->ifs_name, strlen(ndev->ifs_name));

    strncpy(p2->ifi_name, ndev->ifs_name, strlen(ndev->ifs_name));

    int ret = 0;

    ret = get_if_dbytes(p1);

    if (ret < 0)
        goto end;

    usleep(ndev->ifs_us);

    ret = get_if_dbytes(p2);

    if (ret < 0)
        goto end;

    ndev->ifs_ispeed = p2->ifi_ibytes - p1->ifi_ibytes;

    ndev->ifs_ospeed = p2->ifi_obytes - p1->ifi_obytes;

end:

    free(p1);

    free(p2);

    return 0;
}
